#!/usr/bin/env python3

# Code example to show how to use the pyOpenSSL python wrapper to the
# OpenSSL library to download TLS certificates from SSL/TLS servers.
#
# For more information about the lecture go here:
#
#    https://teaching.leonardotamiano.xyz/cns/labs/lab_05_pki
#
# To make the code works we only need to install the pyOpenSSL library
# 
#     pip3 install pyOpenSSL
#
# To use the code simply do
#
#     python3 example_1_tls_certs.py google.com 443
#

import sys
import os
import socket
import json
import OpenSSL.crypto

def dump_cert_data(x509):
    cert_data = {}
    cert_data["subject"] = x509.get_subject().commonName

    pubkey = x509.get_pubkey()
    pubkey_public_params = pubkey.to_cryptography_key().public_numbers()

    if pubkey.type() == OpenSSL.crypto.TYPE_EC:
        cert_data["pubkey"] = {
            "type": "Elliptic-Curve (EC)",
            "curve": pubkey_public_params.curve.name,
            "x": pubkey_public_params.x,
            "y": pubkey_public_params.y,
        }
    elif pubkey.type() == OpenSSL.crypto.TYPE_RSA:
        cert_data["pubkey"] = {
            "type": "RSA",
            "n": pubkey_public_params.n,
            "e": pubkey_public_params.e,
        }

    cert_data["issuer"] = x509.get_issuer().commonName

    return cert_data

def establish_ssl_session(host, port=443):
    dst = (host.encode("utf-8"), port)
    s = socket.create_connection(dst)
    ctx = OpenSSL.SSL.Context(OpenSSL.SSL.SSLv23_METHOD)
    s = OpenSSL.SSL.Connection(ctx, s)
    s.set_connect_state()
    # -- sets the server-name-indication (sni) extension
    s.set_tlsext_host_name(dst[0]) 
    s.do_handshake()
    return s

def get_tls_certs_chain(ssl_session):
    result = {}
    result["host"] = ssl_session.get_servername().decode("utf-8")
    result["tlsVersion"] = ssl_session.get_protocol_version_name()
    for (idx, cert) in enumerate(ssl_session.get_peer_cert_chain()):
        result[str(idx)] = dump_cert_data(cert)
    return result

def main():
    if len(sys.argv) < 3:
        print(f"Usage: {sys.argv[0]} <host> <port>")
        exit()
    try:
        host = sys.argv[1]
        port = int(sys.argv[2])
    except ValueError as e:
        print(f"[ERR] – Failed to convert '{sys.argv[2]}' into an int")
        print(e)
        exit()

    ssl_session = establish_ssl_session(host, port)
    res = get_tls_certs_chain(ssl_session)
    print(json.dumps(res, indent=4))

if __name__ == "__main__":
    main()
