#!/usr/bin/env python3

# Code example to show how to use the pyOpenSSL python wrapper to the
# OpenSSL library to use the Keying Material Exporters offered by the
# SSL/TLS protocol and described in RFC 5705. The exporters allow for
# application using an underlying TLS connection to use the state of
# the TLS session to generate secure and random keys. 
#
# For more information about the lecture go here:
#
#    https://teaching.leonardotamiano.xyz/cns/labs/lab_05_pki
#
# To make the code works we only need to install the pyOpenSSL library
# 
#     pip3 install pyOpenSSL
#
# To use the code simply do
#
#     python3 example_3_export_keying_material.py google.com 443 16

import sys
import os
import socket
import json
import OpenSSL.crypto

def establish_ssl_session(host, port=443):
    dst = (host.encode("utf-8"), port)
    s = socket.create_connection(dst)
    ctx = OpenSSL.SSL.Context(OpenSSL.SSL.SSLv23_METHOD)
    s = OpenSSL.SSL.Connection(ctx, s)
    s.set_connect_state()
    # -- sets the server-name-indication (sni) extension
    s.set_tlsext_host_name(dst[0]) 
    s.do_handshake()
    return s

def get_keying_material(ssl_session, length, label=b"EXPORTER_EXPERIMENTAL", context=None):
    returned_bytes = ssl_session.export_keying_material(label, length, context)
    return returned_bytes

def main():
    if len(sys.argv) < 4:
        print(f"Usage: {sys.argv[0]} <host> <port> <length>")
        exit()
    try:
        host = sys.argv[1]
        port = int(sys.argv[2])
        length = int(sys.argv[3])        
    except ValueError as e:
        print(f"[ERR] – Failed to convert argument into an int")
        print(e)
        exit()

    ssl_session = establish_ssl_session(host, port)
    keying_material = get_keying_material(ssl_session, length)
    if not keying_material:
        print("[ERR] – Failed to export keying material")
        exit()
    print(f"From TLS connection to ({host}, {port}) computed following keying material:")
    print(f"  Bytes: {keying_material}")
    print(f"  Hex: {keying_material.hex()}")

if __name__ == "__main__":
    main()
